@@ -1,5 +1,5 @@
-/**
- * XMLSec library
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * X509 support
  *
@@ -173,21 +173,13 @@ xmlSecGnuTLSX509CertDup(gnutls_x509_crt_t src) {
 
     buf = xmlSecGnuTLSX509CertBase64DerWrite(src, 0);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGnuTLSX509CertBase64DerWrite",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGnuTLSX509CertBase64DerWrite", NULL);
         return (NULL);
     }
 
     res = xmlSecGnuTLSX509CertBase64DerRead(buf);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGnuTLSX509CertBase64DerRead",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGnuTLSX509CertBase64DerRead", NULL);
         xmlFree(buf);
         return (NULL);
     }
@@ -208,33 +200,21 @@ xmlSecGnuTLSX509CertGetSubjectDN(gnutls_x509_crt_t cert) {
     /* get subject size */
     err = gnutls_x509_crt_get_dn(cert, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_dn", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (char *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* finally write it out */
     err = gnutls_x509_crt_get_dn(cert, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_dn", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -254,33 +234,21 @@ xmlSecGnuTLSX509CertGetIssuerDN(gnutls_x509_crt_t cert) {
     /* get issuer size */
     err = gnutls_x509_crt_get_issuer_dn(cert, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_issuer_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_issuer_dn", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (char *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* finally write it out */
     err = gnutls_x509_crt_get_issuer_dn(cert, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_issuer_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_issuer_dn", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -301,33 +269,21 @@ xmlSecGnuTLSX509CertGetIssuerSerial(gnutls_x509_crt_t cert) {
     /* get issuer serial size */
     err = gnutls_x509_crt_get_serial(cert, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_serial",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_serial", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (unsigned char *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* write it out */
     err = gnutls_x509_crt_get_serial(cert, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_serial",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_serial", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -335,11 +291,7 @@ xmlSecGnuTLSX509CertGetIssuerSerial(gnutls_x509_crt_t cert) {
     /* convert to string */
     res = xmlSecGnuTLSASN1IntegerWrite(buf, bufSize);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGnuTLSASN1IntegerWrite",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGnuTLSASN1IntegerWrite", NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -362,33 +314,21 @@ xmlSecGnuTLSX509CertGetSKI(gnutls_x509_crt_t cert) {
     /* get ski size */
     err = gnutls_x509_crt_get_subject_key_id(cert, NULL, &bufSize, &critical);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_subject_key_id",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_subject_key_id", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* write it out */
     err = gnutls_x509_crt_get_subject_key_id(cert, buf, &bufSize, &critical);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_get_subject_key_id",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_get_subject_key_id", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -396,11 +336,7 @@ xmlSecGnuTLSX509CertGetSKI(gnutls_x509_crt_t cert) {
     /* convert to string */
     res = xmlSecBase64Encode(buf, bufSize, 0);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Encode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Encode", NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -420,11 +356,7 @@ xmlSecGnuTLSX509CertBase64DerRead(xmlChar* buf) {
     /* usual trick with base64 decoding "in-place" */
     ret = xmlSecBase64Decode(buf, (xmlSecByte*)buf, xmlStrlen(buf));
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Decode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Decode", NULL);
         return(NULL);
     }
 
@@ -452,22 +384,15 @@ xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFo
         fmt = GNUTLS_X509_FMT_DER;
         break;
     default:
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_FORMAT,
-                    "format=%d", format);
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+                         "format=%d", (int)format);
         return(NULL);
     }
 
     /* read cert */
     err = gnutls_x509_crt_init(&cert);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_init",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_init", err, NULL);
         return(NULL);
     }
 
@@ -475,11 +400,7 @@ xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFo
     data.size = size;
     err = gnutls_x509_crt_import(cert, &data, fmt);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_import",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_import", err, NULL);
         gnutls_x509_crt_deinit(cert);
         return(NULL);
     }
@@ -499,33 +420,21 @@ xmlSecGnuTLSX509CertBase64DerWrite(gnutls_x509_crt_t cert, int base64LineWrap) {
     /* get size */
     err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* write it out */
     err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -533,11 +442,7 @@ xmlSecGnuTLSX509CertBase64DerWrite(gnutls_x509_crt_t cert, int base64LineWrap) {
     /* convert to string */
     res = xmlSecBase64Encode(buf, bufSize, base64LineWrap);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Encode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Encode", NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -628,21 +533,13 @@ xmlSecGnuTLSX509CrlDup(gnutls_x509_crl_t src) {
 
     buf = xmlSecGnuTLSX509CrlBase64DerWrite(src, 0);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGnuTLSX509CrlBase64DerWrite",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGnuTLSX509CrlBase64DerWrite", NULL);
         return (NULL);
     }
 
     res = xmlSecGnuTLSX509CrlBase64DerRead(buf);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGnuTLSX509CrlBase64DerRead",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGnuTLSX509CrlBase64DerRead", NULL);
         xmlFree(buf);
         return (NULL);
     }
@@ -663,33 +560,21 @@ xmlSecGnuTLSX509CrlGetIssuerDN(gnutls_x509_crl_t crl) {
     /* get issuer size */
     err = gnutls_x509_crl_get_issuer_dn(crl, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_get_issuer_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_get_issuer_dn", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (char *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* finally write it out */
     err = gnutls_x509_crl_get_issuer_dn(crl, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_get_issuer_dn",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_get_issuer_dn", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -707,11 +592,7 @@ xmlSecGnuTLSX509CrlBase64DerRead(xmlChar* buf) {
     /* usual trick with base64 decoding "in-place" */
     ret = xmlSecBase64Decode(buf, (xmlSecByte*)buf, xmlStrlen(buf));
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Decode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Decode", NULL);
         return(NULL);
     }
 
@@ -739,22 +620,15 @@ xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFor
         fmt = GNUTLS_X509_FMT_DER;
         break;
     default:
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_FORMAT,
-                    "format=%d", format);
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+                         "format=%d", (int)format);
         return(NULL);
     }
 
     /* read crl */
     err = gnutls_x509_crl_init(&crl);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_init",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_init", err, NULL);
         return(NULL);
     }
 
@@ -762,11 +636,7 @@ xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFor
     data.size = size;
     err = gnutls_x509_crl_import(crl, &data, fmt);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_import",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_import", err, NULL);
         gnutls_x509_crl_deinit(crl);
         return(NULL);
     }
@@ -786,33 +656,21 @@ xmlSecGnuTLSX509CrlBase64DerWrite(gnutls_x509_crl_t crl, int base64LineWrap) {
     /* get size */
     err = gnutls_x509_crl_export(crl, GNUTLS_X509_FMT_DER, NULL, &bufSize);
     if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
         return(NULL);
     }
 
     /* allocate buffer */
     buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
     if(buf == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)bufSize);
+        xmlSecMallocError(bufSize + 1, NULL);
         return(NULL);
     }
 
     /* write it out */
     err = gnutls_x509_crl_export(crl, GNUTLS_X509_FMT_DER, buf, &bufSize);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -820,11 +678,7 @@ xmlSecGnuTLSX509CrlBase64DerWrite(gnutls_x509_crl_t crl, int base64LineWrap) {
     /* convert to string */
     res = xmlSecBase64Encode(buf, bufSize, base64LineWrap);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Encode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Encode", NULL);
         xmlFree(buf);
         return(NULL);
     }
@@ -878,6 +732,7 @@ xmlSecGnuTLSASN1IntegerWrite(const unsigned char * data, size_t len) {
     unsigned long long int val = 0;
     size_t ii = 0;
     int shift = 0;
+    int ret;
 
     xmlSecAssert2(data != NULL, NULL);
     xmlSecAssert2(len <= 9, NULL);
@@ -889,15 +744,17 @@ xmlSecGnuTLSASN1IntegerWrite(const unsigned char * data, size_t len) {
 
     res = (xmlChar*)xmlMalloc(resLen + 1);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)resLen);
-        return (NULL);
+        xmlSecMallocError(resLen + 1, NULL);
+        return(NULL);
+    }
+
+    ret = xmlStrPrintf(res, resLen, "%llu", val);
+    if(ret < 0) {
+        xmlSecXmlError("xmlStrPrintf", NULL);
+        xmlFree(res);
+        return(NULL);
     }
 
-    xmlSecStrPrintf(res, resLen, BAD_CAST "%llu", val);
     return(res);
 }
 
@@ -934,11 +791,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     /* read pkcs12 in internal structure */
     err = gnutls_pkcs12_init(&pkcs12);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_pkcs12_init",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_pkcs12_init", err, NULL);
         goto done;
     }
 
@@ -946,22 +799,14 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     datum.size = dataSize;
     err = gnutls_pkcs12_import(pkcs12, &datum, GNUTLS_X509_FMT_DER, 0);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_pkcs12_import",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_pkcs12_import", err, NULL);
         goto done;
     }
 
     /* verify */
     err = gnutls_pkcs12_verify_mac(pkcs12, pwd);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_pkcs12_verify_mac",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+        xmlSecGnuTLSError("gnutls_pkcs12_verify_mac", err, NULL);
         goto done;
     }
 
@@ -973,11 +818,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
 
         err = gnutls_pkcs12_bag_init(&bag);
         if(err != GNUTLS_E_SUCCESS) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gnutls_pkcs12_bag_init",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GNUTLS_REPORT_ERROR(err));
+            xmlSecGnuTLSError("gnutls_pkcs12_bag_init", err, NULL);
             goto done;
         }
 
@@ -986,32 +827,20 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
             /* scanned the whole pkcs12, stop */
             break;
         } else if(err != GNUTLS_E_SUCCESS) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gnutls_pkcs12_get_bag",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GNUTLS_REPORT_ERROR(err));
+            xmlSecGnuTLSError("gnutls_pkcs12_get_bag", err, NULL);
             goto done;
         }
 
         /* check if we need to decrypt the bag */
         bag_type = gnutls_pkcs12_bag_get_type(bag, 0);
         if(bag_type < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gnutls_pkcs12_bag_get_type",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GNUTLS_REPORT_ERROR(bag_type));
+            xmlSecGnuTLSError("gnutls_pkcs12_bag_get_type", bag_type, NULL);
             goto done;
         }
         if(bag_type == GNUTLS_BAG_ENCRYPTED) {
             err = gnutls_pkcs12_bag_decrypt(bag, pwd);
             if(err != GNUTLS_E_SUCCESS) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "gnutls_pkcs12_bag_decrypt",
-                            XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                            XMLSEC_GNUTLS_REPORT_ERROR(err));
+                xmlSecGnuTLSError("gnutls_pkcs12_bag_decrypt", err, NULL);
                 goto done;
             }
         }
@@ -1019,31 +848,19 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
         /* scan elements in bag */
         elements_in_bag = gnutls_pkcs12_bag_get_count(bag);
         if(elements_in_bag < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gnutls_pkcs12_bag_get_count",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GNUTLS_REPORT_ERROR(bag_type));
+            xmlSecGnuTLSError("gnutls_pkcs12_bag_get_count", elements_in_bag, NULL);
             goto done;
         }
         for(ii = 0; ii < elements_in_bag; ++ii) {
             bag_type = gnutls_pkcs12_bag_get_type(bag, ii);
             if(bag_type < 0) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "gnutls_pkcs12_bag_get_type",
-                            XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                            XMLSEC_GNUTLS_REPORT_ERROR(bag_type));
+                xmlSecGnuTLSError("gnutls_pkcs12_bag_get_type", bag_type, NULL);
                 goto done;
             }
 
             err = gnutls_pkcs12_bag_get_data(bag, ii, &datum);
             if(err != GNUTLS_E_SUCCESS) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "gnutls_pkcs12_bag_get_data",
-                            XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                            XMLSEC_GNUTLS_REPORT_ERROR(err));
+                xmlSecGnuTLSError("gnutls_pkcs12_bag_get_data", err, NULL);
                 goto done;
             }
 
@@ -1054,11 +871,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
                 if((*priv_key) == NULL) {
                     err = gnutls_x509_privkey_init(priv_key);
                     if(err != GNUTLS_E_SUCCESS) {
-                        xmlSecError(XMLSEC_ERRORS_HERE,
-                                    NULL,
-                                    "gnutls_x509_privkey_init",
-                                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+                        xmlSecGnuTLSError("gnutls_x509_privkey_init", err, NULL);
                         goto done;
                     }
 
@@ -1067,11 +880,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
                                 pwd,
                                 (bag_type == GNUTLS_BAG_PKCS8_KEY) ? GNUTLS_PKCS_PLAIN : 0);
                     if(err != GNUTLS_E_SUCCESS) {
-                        xmlSecError(XMLSEC_ERRORS_HERE,
-                                    NULL,
-                                    "gnutls_x509_privkey_import_pkcs8",
-                                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                                    XMLSEC_GNUTLS_REPORT_ERROR(err));
+                        xmlSecGnuTLSError("gnutls_x509_privkey_import_pkcs8", err, NULL);
                         goto done;
                     }
                 }
@@ -1079,31 +888,19 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
             case GNUTLS_BAG_CERTIFICATE:
                 err = gnutls_x509_crt_init(&cert);
                 if(err != GNUTLS_E_SUCCESS) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "gnutls_x509_crt_init",
-                                XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                                XMLSEC_GNUTLS_REPORT_ERROR(err));
+                    xmlSecGnuTLSError("gnutls_x509_crt_init", err, NULL);
                     goto done;
                 }
 
                 err = gnutls_x509_crt_import(cert, &datum, GNUTLS_X509_FMT_DER);
                 if(err != GNUTLS_E_SUCCESS) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "gnutls_x509_crt_import",
-                                XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                                XMLSEC_GNUTLS_REPORT_ERROR(err));
+                    xmlSecGnuTLSError("gnutls_x509_crt_import", err, NULL);
                     goto done;
                 }
 
                 ret = xmlSecPtrListAdd(certsList, cert);
                 if(ret < 0) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "xmlSecPtrListAdd(certsList)",
-                                XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                                XMLSEC_ERRORS_NO_MESSAGE);
+                    xmlSecInternalError("xmlSecPtrListAdd(certsList)", NULL);
                     goto done;
                 }
                 cert = NULL; /* owned by certsList now */
@@ -1121,11 +918,8 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
 
     /* check we have private key */
     if((*priv_key) == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "Private key was not found in pkcs12 object");
+        xmlSecOtherError(XMLSEC_ERRORS_R_KEY_NOT_FOUND, NULL,
+                         "Private key was not found in pkcs12 object");
         goto done;
     }
 
@@ -1141,11 +935,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
         key_id_size = sizeof(key_id);
         err = gnutls_x509_privkey_get_key_id((*priv_key), 0, key_id, &key_id_size);
         if(err != GNUTLS_E_SUCCESS) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gnutls_x509_privkey_get_key_id",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GNUTLS_REPORT_ERROR(err));
+            xmlSecGnuTLSError("gnutls_x509_privkey_get_key_id", err, NULL);
             goto done;
         }
         for(ii = 0; ii < certsSize; ++ii) {
@@ -1159,11 +949,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
             cert_id_size = sizeof(cert_id);
             err = gnutls_x509_crt_get_key_id(tmp, 0, cert_id, &cert_id_size);
             if(err != GNUTLS_E_SUCCESS) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "gnutls_x509_crt_get_key_id",
-                            XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                            XMLSEC_GNUTLS_REPORT_ERROR(err));
+                xmlSecGnuTLSError("gnutls_x509_crt_get_key_id", err, NULL);
                 goto done;
             }
 
@@ -1171,11 +957,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
             if((key_id_size == cert_id_size) && (memcmp(key_id, cert_id, key_id_size) == 0)) {
                 (*key_cert) = xmlSecGnuTLSX509CertDup(tmp);
                 if((*key_cert) == NULL) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "xmlSecGnuTLSX509CertDup",
-                                XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                                XMLSEC_ERRORS_NO_MESSAGE);
+                    xmlSecInternalError("xmlSecGnuTLSX509CertDup", NULL);
                     goto done;
                 }
 
@@ -1185,11 +967,8 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
 
         /* check we have key cert */
         if((*key_cert) == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        NULL,
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "Certificate for the private key was not found in pkcs12 object");
+            xmlSecOtherError(XMLSEC_ERRORS_R_CERT_NOT_FOUND, NULL,
+                             "Certificate for the private key was not found in pkcs12 object");
             goto done;
         }
     }
@@ -1222,11 +1001,7 @@ xmlSecGnuTLSCreateKeyDataAndAdoptPrivKey(gnutls_x509_privkey_t priv_key) {
     /* create key value data */
     key_alg = gnutls_x509_privkey_get_pk_algorithm(priv_key);
     if(key_alg < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_privkey_get_pk_algorithm",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_GNUTLS_REPORT_ERROR(key_alg));
+        xmlSecGnuTLSError("gnutls_x509_privkey_get_pk_algorithm", key_alg < 0, NULL);
         return (NULL);
     }
     switch(key_alg) {
@@ -1234,21 +1009,13 @@ xmlSecGnuTLSCreateKeyDataAndAdoptPrivKey(gnutls_x509_privkey_t priv_key) {
     case GNUTLS_PK_RSA:
         res = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataRsaId);
         if(res == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGnuTLSKeyDataRsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataRsaId)", NULL);
             return(NULL);
         }
 
         ret = xmlSecGnuTLSKeyDataRsaAdoptPrivateKey(res, priv_key);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGnuTLSKeyDataRsaAdoptPrivateKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGnuTLSKeyDataRsaId");
+            xmlSecInternalError("xmlSecGnuTLSKeyDataRsaAdoptPrivateKey(KeyDataRsaId)", NULL);
             xmlSecKeyDataDestroy(res);
             return(NULL);
         }
@@ -1259,32 +1026,20 @@ xmlSecGnuTLSCreateKeyDataAndAdoptPrivKey(gnutls_x509_privkey_t priv_key) {
     case GNUTLS_PK_DSA:
         res = xmlSecKeyDataCreate(xmlSecGnuTLSKeyDataDsaId);
         if(res == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGnuTLSKeyDataDsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(KeyDataDsaId)", NULL);
             return(NULL);
         }
 
         ret = xmlSecGnuTLSKeyDataDsaAdoptPrivateKey(res, priv_key);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGnuTLSKeyDataDsaAdoptPrivateKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGnuTLSKeyDataDsaId");
+            xmlSecInternalError("xmlSecGnuTLSKeyDataDsaAdoptPrivateKey(KeyDataDsaId)", NULL);
             xmlSecKeyDataDestroy(res);
             return(NULL);
         }
         break;
 #endif /* XMLSEC_NO_DSA */
     default:
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "gnutls_x509_privkey_get_pk_algorithm",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "Unsupported algorithm %d", (int)key_alg);
+        xmlSecInvalidIntegerTypeError("key_alg", key_alg, "supported algorithm", NULL);
         return(NULL);
     }
 
@@ -1493,11 +1248,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
     /* allocate buffer, we don't need more than string */
     tmp = (xmlChar *)xmlMalloc(xmlStrlen(dn) + 1);
     if(tmp == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlMalloc",
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", (int)(xmlStrlen(dn) + 1));
+        xmlSecMallocError(xmlStrlen(dn) + 1, NULL);
         goto done;
     }
 
@@ -1532,20 +1283,13 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
 
                 /* insert into the attrs */
                 if(pos >= attrsSize) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "",
-                                XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                                "Not enough space: size=%d", (int)attrsSize);
+                    xmlSecInvalidSizeLessThanError("Attributes",
+                                                   attrsSize, pos, NULL);
                     goto done;
                 }
                 attrs[pos].key = xmlStrdup(tmp);
                 if(attrs[pos].key == NULL) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "xmlStrdup",
-                                XMLSEC_ERRORS_R_MALLOC_FAILED,
-                                "size=%d", (int)(xmlStrlen(tmp) + 1));
+                    xmlSecStrdupError(tmp, NULL);
                     goto done;
                 }
 
@@ -1583,11 +1327,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
 
                 attrs[pos].value = xmlStrdup(tmp);
                 if(attrs[pos].value == NULL) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "xmlStrdup",
-                                XMLSEC_ERRORS_R_MALLOC_FAILED,
-                                "size=%d", (int)(xmlStrlen(tmp) + 1));
+                    xmlSecStrdupError(tmp, NULL);
                     goto done;
                 }
                 state = xmlSecGnuTLSDnParseState_BeforeNameComponent;
@@ -1609,11 +1349,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
 
                 attrs[pos].value = xmlStrdup(tmp);
                 if(attrs[pos].value == NULL) {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "xmlStrdup",
-                                XMLSEC_ERRORS_R_MALLOC_FAILED,
-                                "size=%d", (int)(xmlStrlen(tmp) + 1));
+                    xmlSecStrdupError(tmp, NULL);
                     goto done;
                 }
                 state = xmlSecGnuTLSDnParseState_AfterQuotedString;
@@ -1628,12 +1364,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
                 if((ch == ',') || (ch == ';') || (ch == '\0')) {
                     state = xmlSecGnuTLSDnParseState_BeforeNameComponent;
                 } else {
-                    xmlSecError(XMLSEC_ERRORS_HERE,
-                                NULL,
-                                "",
-                                XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                                "Unexpected character %c (expected space or ',' or ';')",
-                                ch);
+                    xmlSecInvalidIntegerDataError("ch", ch, "space,',',';','\\0'", NULL);
                     goto done;
                 }
             } else {
@@ -1650,12 +1381,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
 
     /* check end state */
     if(state != xmlSecGnuTLSDnParseState_BeforeNameComponent) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "Unexpected state %d at the end of parsing",
-                    (int)state);
+        xmlSecInvalidIntegerDataError("state", state, "xmlSecGnuTLSDnParseState_BeforeNameComponent", NULL);
         goto done;
     }
 
