@@ -1,10 +1,10 @@
-/**
- * XMLSec library
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #include "globals.h"
 
@@ -20,11 +20,8 @@
 
 #include <xmlsec/openssl/crypto.h>
 #include <xmlsec/openssl/evp.h>
+#include "openssl_compat.h"
 
-/* this is not defined in OpenSSL 0.9.6 */
-#ifndef EVP_MAX_BLOCK_LENGTH
-#define EVP_MAX_BLOCK_LENGTH            32
-#endif /* EVP_MAX_BLOCK_LENGTH */
 
 /**************************************************************************
  *
@@ -36,25 +33,33 @@ typedef struct _xmlSecOpenSSLEvpBlockCipherCtx          xmlSecOpenSSLEvpBlockCip
 struct _xmlSecOpenSSLEvpBlockCipherCtx {
     const EVP_CIPHER*   cipher;
     xmlSecKeyDataId     keyId;
-    EVP_CIPHER_CTX      cipherCtx;
+    EVP_CIPHER_CTX*     cipherCtx;
     int                 keyInitialized;
     int                 ctxInitialized;
     xmlSecByte          key[EVP_MAX_KEY_LENGTH];
     xmlSecByte          iv[EVP_MAX_IV_LENGTH];
-    xmlSecByte          pad[EVP_MAX_BLOCK_LENGTH];
+    xmlSecByte          pad[2*EVP_MAX_BLOCK_LENGTH];
 };
+
 static int      xmlSecOpenSSLEvpBlockCipherCtxInit      (xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
                                                          xmlSecBufferPtr in,
                                                          xmlSecBufferPtr out,
                                                          int encrypt,
                                                          const xmlChar* cipherName,
                                                          xmlSecTransformCtxPtr transformCtx);
+static int      xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
+                                                         const xmlSecByte * in,
+                                                         int inSize,
+                                                         xmlSecBufferPtr out,
+                                                         const xmlChar* cipherName,
+                                                         int final);
 static int      xmlSecOpenSSLEvpBlockCipherCtxUpdate    (xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
                                                          xmlSecBufferPtr in,
                                                          xmlSecBufferPtr out,
                                                          const xmlChar* cipherName,
                                                          xmlSecTransformCtxPtr transformCtx);
 static int      xmlSecOpenSSLEvpBlockCipherCtxFinal     (xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
+                                                         xmlSecBufferPtr in,
                                                          xmlSecBufferPtr out,
                                                          const xmlChar* cipherName,
                                                          xmlSecTransformCtxPtr transformCtx);
@@ -69,6 +74,7 @@ xmlSecOpenSSLEvpBlockCipherCtxInit(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
 
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->cipher != NULL, -1);
+    xmlSecAssert2(ctx->cipherCtx != NULL, -1);
     xmlSecAssert2(ctx->keyInitialized != 0, -1);
     xmlSecAssert2(ctx->ctxInitialized == 0, -1);
     xmlSecAssert2(in != NULL, -1);
@@ -83,22 +89,15 @@ xmlSecOpenSSLEvpBlockCipherCtxInit(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
         /* generate random iv */
         ret = RAND_bytes(ctx->iv, ivLen);
         if(ret != 1) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        xmlSecErrorsSafeString(cipherName),
-                        "RAND_bytes",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        "size=%d", ivLen);
+            xmlSecOpenSSLError2("RAND_bytes", cipherName,
+                                "size=%lu", (unsigned long)ivLen);
             return(-1);
         }
 
         /* write iv to the output */
         ret = xmlSecBufferAppend(out, ctx->iv, ivLen);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        xmlSecErrorsSafeString(cipherName),
-                        "xmlSecBufferAppend",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "size=%d", ivLen);
+            xmlSecInternalError2("xmlSecBufferAppend", cipherName, "size=%d", ivLen);
             return(-1);
         }
 
@@ -116,23 +115,15 @@ xmlSecOpenSSLEvpBlockCipherCtxInit(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
         /* and remove from input */
         ret = xmlSecBufferRemoveHead(in, ivLen);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        xmlSecErrorsSafeString(cipherName),
-                        "xmlSecBufferRemoveHead",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "size=%d", ivLen);
+            xmlSecInternalError2("xmlSecBufferRemoveHead", cipherName, "size=%d", ivLen);
             return(-1);
         }
     }
 
     /* set iv */
-    ret = EVP_CipherInit(&(ctx->cipherCtx), ctx->cipher, ctx->key, ctx->iv, encrypt);
+    ret = EVP_CipherInit(ctx->cipherCtx, ctx->cipher, ctx->key, ctx->iv, encrypt);
     if(ret != 1) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "EVP_CipherInit",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecOpenSSLError("EVP_CipherIn", cipherName);
         return(-1);
     }
 
@@ -140,27 +131,99 @@ xmlSecOpenSSLEvpBlockCipherCtxInit(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
 
     /*
      * The padding used in XML Enc does not follow RFC 1423
-     * and is not supported by OpenSSL. In the case of OpenSSL 0.9.7
-     * it is possible to disable padding and do it by yourself
-     * For OpenSSL 0.9.6 you have interop problems
+     * and is not supported by OpenSSL. However, it is possible
+     * to disable padding and do it by yourself
+     *
+     * https://www.w3.org/TR/2002/REC-xmlenc-core-20021210/Overview.html#sec-Alg-Block
      */
-#ifndef XMLSEC_OPENSSL_096
-    EVP_CIPHER_CTX_set_padding(&(ctx->cipherCtx), 0);
-#endif /* XMLSEC_OPENSSL_096 */
+    EVP_CIPHER_CTX_set_padding(ctx->cipherCtx, 0);
+
     return(0);
 }
 
+static int
+xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
+                                        const xmlSecByte * in,
+                                        int inSize,
+                                        xmlSecBufferPtr out,
+                                        const xmlChar* cipherName,
+                                        int final) {
+    xmlSecByte* outBuf;
+    xmlSecSize outSize;
+    int blockLen, outLen = 0;
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->cipher != NULL, -1);
+    xmlSecAssert2(ctx->cipherCtx != NULL, -1);
+    xmlSecAssert2(ctx->keyInitialized != 0, -1);
+    xmlSecAssert2(ctx->ctxInitialized != 0, -1);
+    xmlSecAssert2(in != NULL, -1);
+    xmlSecAssert2(inSize > 0, -1);
+    xmlSecAssert2(out != NULL, -1);
+
+    /* OpenSSL docs: If the pad parameter is zero then no padding is performed, the total amount of
+     * data encrypted or decrypted must then be a multiple of the block size or an error will occur.
+     */
+    blockLen = EVP_CIPHER_block_size(ctx->cipher);
+    xmlSecAssert2(blockLen > 0, -1);
+    xmlSecAssert2((inSize % blockLen) == 0, -1);
+
+    /* prepare: ensure we have enough space (+blockLen for final) */
+    outSize = xmlSecBufferGetSize(out);
+    ret = xmlSecBufferSetMaxSize(out, outSize + inSize + blockLen);
+    if(ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetMaxSize",
+                             xmlSecErrorsSafeString(cipherName),
+                            "size=%d", (int)(outSize + inSize + blockLen));
+        return(-1);
+    }
+    outBuf  = xmlSecBufferGetData(out) + outSize;
+
+    /* encrypt/decrypt */
+    ret = EVP_CipherUpdate(ctx->cipherCtx, outBuf, &outLen, in, inSize);
+    if(ret != 1) {
+        xmlSecOpenSSLError("EVP_CipherUpdate", cipherName);
+        return(-1);
+    }
+    xmlSecAssert2(outLen == inSize, -1);
+
+    /* finalize transform if needed */
+    if(final != 0) {
+        int outLen2 = 0;
+
+        ret = EVP_CipherFinal(ctx->cipherCtx, outBuf + outLen, &outLen2);
+        if(ret != 1) {
+            xmlSecOpenSSLError("EVP_CipherFinal", cipherName);
+            return(-1);
+        }
+
+        outLen += outLen2;
+    }
+
+    /* set correct output buffer size */
+    ret = xmlSecBufferSetSize(out, outSize + outLen);
+    if(ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetSize", cipherName,
+                             "size=%d", (int)(outSize + outLen));
+        return(-1);
+    }
+
+    /* done */
+    return (0);
+}
+
 static int
 xmlSecOpenSSLEvpBlockCipherCtxUpdate(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
                                   xmlSecBufferPtr in, xmlSecBufferPtr out,
                                   const xmlChar* cipherName,
                                   xmlSecTransformCtxPtr transformCtx) {
-    int blockLen, fixLength = 0, outLen = 0;
-    xmlSecSize inSize, outSize;
-    xmlSecByte* outBuf;
+    xmlSecSize inSize, blockLen, inBlocksLen;
+    xmlSecByte* inBuf;
     int ret;
 
     xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->cipherCtx != NULL, -1);
     xmlSecAssert2(ctx->keyInitialized != 0, -1);
     xmlSecAssert2(ctx->ctxInitialized != 0, -1);
     xmlSecAssert2(in != NULL, -1);
@@ -171,250 +234,161 @@ xmlSecOpenSSLEvpBlockCipherCtxUpdate(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
     xmlSecAssert2(blockLen > 0, -1);
 
     inSize = xmlSecBufferGetSize(in);
-    outSize = xmlSecBufferGetSize(out);
-
-    if(inSize == 0) {
-        /* wait for more data */
+    if(inSize <= blockLen) {
+        /* wait for more data: we want to make sure we keep the last chunk in tmp buffer for
+         * padding check/removal on decryption
+         */
         return(0);
     }
 
-    /* OpenSSL docs: The amount of data written depends on the block
-     * alignment of the encrypted data: as a result the amount of data
-     * written may be anything from zero bytes to (inl + cipher_block_size - 1).
-     */
-    ret = xmlSecBufferSetMaxSize(out, outSize + inSize + blockLen);
-    if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "xmlSecBufferSetMaxSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", outSize + inSize + blockLen);
-        return(-1);
-    }
-    outBuf = xmlSecBufferGetData(out) + outSize;
-
-    /*
-     * The padding used in XML Enc does not follow RFC 1423
-     * and is not supported by OpenSSL. In the case of OpenSSL 0.9.7
-     * it is possible to disable padding and do it by yourself
-     * For OpenSSL 0.9.6 you have interop problems.
+    /* OpenSSL docs: If the pad parameter is zero then no padding is performed, the total amount of
+     * data encrypted or decrypted must then be a multiple of the block size or an error will occur.
      *
-     * The logic below is copied from EVP_DecryptUpdate() function.
-     * This is a hack but it's the only way I can provide binary
-     * compatibility with previous versions of xmlsec.
-     * This needs to be fixed in the next XMLSEC API refresh.
+     * We process all complete blocks from the input
      */
-#ifndef XMLSEC_OPENSSL_096
-    if(!ctx->cipherCtx.encrypt) {
-        if(ctx->cipherCtx.final_used) {
-            memcpy(outBuf, ctx->cipherCtx.final, blockLen);
-            outBuf += blockLen;
-            fixLength = 1;
-        } else {
-            fixLength = 0;
-        }
+    inBlocksLen = blockLen * (inSize / blockLen);
+    if(inBlocksLen == inSize) {
+        inBlocksLen -= blockLen; /* ensure we keep the last block around for Final() call to add/check/remove padding */
     }
-#endif /* XMLSEC_OPENSSL_096 */
+    xmlSecAssert2(inBlocksLen > 0, -1);
 
-    /* encrypt/decrypt */
-    ret = EVP_CipherUpdate(&(ctx->cipherCtx), outBuf, &outLen, xmlSecBufferGetData(in), inSize);
-    if(ret != 1) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "EVP_CipherUpdate",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
-        return(-1);
-    }
-
-#ifndef XMLSEC_OPENSSL_096
-    if(!ctx->cipherCtx.encrypt) {
-        /*
-         * The logic below is copied from EVP_DecryptUpdate() function.
-         * This is a hack but it's the only way I can provide binary
-         * compatibility with previous versions of xmlsec.
-         * This needs to be fixed in the next XMLSEC API refresh.
-         */
-        if (blockLen > 1 && !ctx->cipherCtx.buf_len) {
-            outLen -= blockLen;
-            ctx->cipherCtx.final_used = 1;
-            memcpy(ctx->cipherCtx.final, &outBuf[outLen], blockLen);
-        } else {
-            ctx->cipherCtx.final_used = 0;
-        }
-        if (fixLength) {
-            outLen += blockLen;
-        }
-    }
-#endif /* XMLSEC_OPENSSL_096 */
-
-    /* set correct output buffer size */
-    ret = xmlSecBufferSetSize(out, outSize + outLen);
+    inBuf  = xmlSecBufferGetData(in);
+    ret = xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(ctx, inBuf, inBlocksLen, out, cipherName, 0); /* not final */
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "xmlSecBufferSetSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", outSize + outLen);
+        xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock", cipherName);
         return(-1);
     }
 
     /* remove the processed block from input */
-    ret = xmlSecBufferRemoveHead(in, inSize);
+    ret = xmlSecBufferRemoveHead(in, inBlocksLen);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "xmlSecBufferRemoveHead",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", inSize);
+        xmlSecInternalError2("xmlSecBufferRemoveHead", cipherName, "size=%d", (int)inSize);
         return(-1);
     }
+
+    /* just a double check */
+    inSize = xmlSecBufferGetSize(in);
+    xmlSecAssert2(inSize > 0, -1);
+    xmlSecAssert2(inSize <= blockLen, -1);
+
+    /* done */
     return(0);
 }
 
 static int
 xmlSecOpenSSLEvpBlockCipherCtxFinal(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
+                                 xmlSecBufferPtr in,
                                  xmlSecBufferPtr out,
                                  const xmlChar* cipherName,
                                  xmlSecTransformCtxPtr transformCtx) {
-    int blockLen, outLen = 0, outLen2 = 0;
-    xmlSecSize outSize;
+    xmlSecSize inSize, outSize, blockLen;
+    xmlSecByte* inBuf;
     xmlSecByte* outBuf;
     int ret;
 
     xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->cipher != NULL, -1);
+    xmlSecAssert2(ctx->cipherCtx != NULL, -1);
     xmlSecAssert2(ctx->keyInitialized != 0, -1);
     xmlSecAssert2(ctx->ctxInitialized != 0, -1);
+    xmlSecAssert2(in != NULL, -1);
     xmlSecAssert2(out != NULL, -1);
     xmlSecAssert2(transformCtx != NULL, -1);
 
     blockLen = EVP_CIPHER_block_size(ctx->cipher);
     xmlSecAssert2(blockLen > 0, -1);
+    xmlSecAssert2(blockLen <= EVP_MAX_BLOCK_LENGTH, -1);
 
-    outSize = xmlSecBufferGetSize(out);
-
-    /* OpenSSL docs: The encrypted final data is written to out which should
-     * have sufficient space for one cipher block. We might have to write
-     * one more block with padding
-     */
-    ret = xmlSecBufferSetMaxSize(out, outSize + 2 * blockLen);
-    if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "xmlSecBufferSetMaxSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", outSize + 2 * blockLen);
-        return(-1);
-    }
-    outBuf = xmlSecBufferGetData(out) + outSize;
+    /* not more than one block left */
+    inSize = xmlSecBufferGetSize(in);
+    inBuf = xmlSecBufferGetData(in);
+    xmlSecAssert2(inSize <= blockLen, -1);
 
     /*
      * The padding used in XML Enc does not follow RFC 1423
-     * and is not supported by OpenSSL. In the case of OpenSSL 0.9.7
-     * it is possible to disable padding and do it by yourself
-     * For OpenSSL 0.9.6 you have interop problems.
+     * and is not supported by OpenSSL. However, it is possible
+     * to disable padding and do it by yourself
      *
-     * The logic below is copied from EVP_DecryptFinal() function.
-     * This is a hack but it's the only way I can provide binary
-     * compatibility with previous versions of xmlsec.
-     * This needs to be fixed in the next XMLSEC API refresh.
+     * https://www.w3.org/TR/2002/REC-xmlenc-core-20021210/Overview.html#sec-Alg-Block
      */
-#ifndef XMLSEC_OPENSSL_096
-    if(ctx->cipherCtx.encrypt) {
-        int padLen;
+    if(EVP_CIPHER_CTX_encrypting(ctx->cipherCtx)) {
+        xmlSecSize padLen;
 
-        xmlSecAssert2(blockLen <= EVP_MAX_BLOCK_LENGTH, -1);
-
-        padLen = blockLen - ctx->cipherCtx.buf_len;
+        /* figure out pad length, if it is 0 (i.e. inSize == blockLen) then set it to blockLen */
+        padLen = blockLen - inSize;
+        if(padLen == 0) {
+            padLen = blockLen;
+        }
         xmlSecAssert2(padLen > 0, -1);
+        xmlSecAssert2(inSize + padLen <= sizeof(ctx->pad), -1);
+
+        /* we can have inSize == 0 if there were no data at all, otherwise -- copy the data */
+        if(inSize > 0) {
+            memcpy(ctx->pad, inBuf, inSize);
+        }
 
         /* generate random padding */
         if(padLen > 1) {
-            ret = RAND_bytes(ctx->pad, padLen - 1);
+            ret = RAND_bytes(ctx->pad + inSize, padLen - 1);
             if(ret != 1) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            xmlSecErrorsSafeString(cipherName),
-                            "RAND_bytes",
-                            XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                            "size=%d", padLen - 1);
+                xmlSecOpenSSLError("RAND_bytes", cipherName);
                 return(-1);
             }
         }
-        ctx->pad[padLen - 1] = padLen;
 
-        /* write padding */
-        ret = EVP_CipherUpdate(&(ctx->cipherCtx), outBuf, &outLen, ctx->pad, padLen);
-        if(ret != 1) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        xmlSecErrorsSafeString(cipherName),
-                        "EVP_CipherUpdate",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+        /* set the last byte to the pad length */
+        ctx->pad[inSize + padLen - 1] = padLen;
+
+        /* update the last 1 or 2 blocks with padding */
+        ret = xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(ctx, ctx->pad, inSize + padLen, out, cipherName, 1); /* final */
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock", cipherName);
             return(-1);
         }
-        outBuf += outLen;
-    }
-#endif /* XMLSEC_OPENSSL_096 */
+    } else {
+        xmlSecSize padLen;
 
-    /* finalize transform */
-    ret = EVP_CipherFinal(&(ctx->cipherCtx), outBuf, &outLen2);
-    if(ret != 1) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "EVP_CipherFinal",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
-        return(-1);
-    }
+        /* update the last one block with padding */
+        ret = xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(ctx, inBuf, inSize, out, cipherName, 1); /* final */
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock", cipherName);
+            return(-1);
+        }
 
-    /*
-     * The padding used in XML Enc does not follow RFC 1423
-     * and is not supported by OpenSSL. In the case of OpenSSL 0.9.7
-     * it is possible to disable padding and do it by yourself
-     * For OpenSSL 0.9.6 you have interop problems.
-     *
-     * The logic below is copied from EVP_DecryptFinal() function.
-     * This is a hack but it's the only way I can provide binary
-     * compatibility with previous versions of xmlsec.
-     * This needs to be fixed in the next XMLSEC API refresh.
-     */
-#ifndef XMLSEC_OPENSSL_096
-     if(!ctx->cipherCtx.encrypt) {
-        /* we instructed openssl to do not use padding so there
-         * should be no final block
-         */
-        xmlSecAssert2(outLen2 == 0, -1);
-        xmlSecAssert2(ctx->cipherCtx.buf_len == 0, -1);
-        xmlSecAssert2(ctx->cipherCtx.final_used, -1);
+        /* we expect at least one block in the output -- the one we just decrypted */
+        outBuf = xmlSecBufferGetData(out);
+        outSize = xmlSecBufferGetSize(out);
+        if(outSize < blockLen) {
+            xmlSecInvalidIntegerDataError2("outSize", outSize, "blockLen", blockLen,
+                    "outSize >= blockLen", cipherName);
+            return(-1);
+        }
 
-        if(blockLen > 1) {
-            outLen2 = blockLen - ctx->cipherCtx.final[blockLen - 1];
-            if(outLen2 > 0) {
-                memcpy(outBuf, ctx->cipherCtx.final, outLen2);
-            } else if(outLen2 < 0) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            xmlSecErrorsSafeString(cipherName),
-                            NULL,
-                            XMLSEC_ERRORS_R_INVALID_DATA,
-                            "padding=%d;buffer=%d",
-                            ctx->cipherCtx.final[blockLen - 1], blockLen);
-                return(-1);
-            }
+        /* get the pad length from the last byte */
+        padLen = (xmlSecSize)(outBuf[outSize - 1]);
+        if(padLen > blockLen) {
+            xmlSecInvalidIntegerDataError2("padLen", padLen, "blockLen", blockLen,
+                    "padLen <= blockLen", cipherName);
+            return(-1);
+        }
+        xmlSecAssert2(padLen <= outSize, -1);
+
+        /* remove the padding */
+        ret = xmlSecBufferRemoveTail(out, padLen);
+        if(ret < 0) {
+            xmlSecInternalError2("xmlSecBufferRemoveTail", cipherName, "size=%d", (int)padLen);
+            return(-1);
         }
     }
-#endif /* XMLSEC_OPENSSL_096 */
 
-    /* set correct output buffer size */
-    ret = xmlSecBufferSetSize(out, outSize + outLen + outLen2);
+    /* remove the processed block from input */
+    ret = xmlSecBufferRemoveHead(in, inSize);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(cipherName),
-                    "xmlSecBufferSetSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", outSize + outLen + outLen2);
+        xmlSecInternalError2("xmlSecBufferRemoveHead", cipherName, "size=%d", (int)inSize);
         return(-1);
     }
 
+    /* done */
     return(0);
 }
 
@@ -497,15 +471,19 @@ xmlSecOpenSSLEvpBlockCipherInitialize(xmlSecTransformPtr transform) {
 #endif /* XMLSEC_NO_AES */
 
     if(1) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_TRANSFORM,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInvalidTransfromError(transform)
         return(-1);
     }
 
-    EVP_CIPHER_CTX_init(&(ctx->cipherCtx));
+    /* create cipher ctx */
+    ctx->cipherCtx = EVP_CIPHER_CTX_new();
+    if(ctx->cipherCtx == NULL) {
+        xmlSecOpenSSLError("EVP_CIPHER_CTX_new",
+                           xmlSecTransformGetName(transform));
+        return(-1);
+    }
+
+    /* done */
     return(0);
 }
 
@@ -519,7 +497,10 @@ xmlSecOpenSSLEvpBlockCipherFinalize(xmlSecTransformPtr transform) {
     ctx = xmlSecOpenSSLEvpBlockCipherGetCtx(transform);
     xmlSecAssert(ctx != NULL);
 
-    EVP_CIPHER_CTX_cleanup(&(ctx->cipherCtx));
+    if(ctx->cipherCtx != NULL) {
+        EVP_CIPHER_CTX_free(ctx->cipherCtx);
+    }
+
     memset(ctx, 0, sizeof(xmlSecOpenSSLEvpBlockCipherCtx));
 }
 
@@ -579,12 +560,8 @@ xmlSecOpenSSLEvpBlockCipherSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key
     xmlSecAssert2(buffer != NULL, -1);
 
     if(xmlSecBufferGetSize(buffer) < (xmlSecSize)cipherKeyLen) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_KEY_DATA_SIZE,
-                    "keySize=%d;expected=%d",
-                    xmlSecBufferGetSize(buffer), cipherKeyLen);
+        xmlSecInvalidKeyDataSizeError(xmlSecBufferGetSize(buffer), cipherKeyLen,
+                xmlSecTransformGetName(transform));
         return(-1);
     }
 
@@ -622,52 +599,41 @@ xmlSecOpenSSLEvpBlockCipherExecute(xmlSecTransformPtr transform, int last, xmlSe
                         (transform->operation == xmlSecTransformOperationEncrypt) ? 1 : 0,
                         xmlSecTransformGetName(transform), transformCtx);
             if(ret < 0) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                            "xmlSecOpenSSLEvpBlockCipherCtxInit",
-                            XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                            XMLSEC_ERRORS_NO_MESSAGE);
+                xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxInit",
+                        xmlSecTransformGetName(transform));
                 return(-1);
             }
         }
         if((ctx->ctxInitialized == 0) && (last != 0)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                        NULL,
-                        XMLSEC_ERRORS_R_INVALID_DATA,
-                        "not enough data to initialize transform");
+            xmlSecInvalidDataError("not enough data to initialize transform",
+                    xmlSecTransformGetName(transform));
             return(-1);
         }
 
         if(ctx->ctxInitialized != 0) {
             ret = xmlSecOpenSSLEvpBlockCipherCtxUpdate(ctx, in, out,
-                                                xmlSecTransformGetName(transform),
-                                                transformCtx);
+                    xmlSecTransformGetName(transform),
+                    transformCtx);
             if(ret < 0) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                            "xmlSecOpenSSLEvpBlockCipherCtxUpdate",
-                            XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                            XMLSEC_ERRORS_NO_MESSAGE);
+                xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxUpdate",
+                        xmlSecTransformGetName(transform));
                 return(-1);
             }
         }
 
         if(last != 0) {
-            /* by now there should be no input */
-            xmlSecAssert2(xmlSecBufferGetSize(in) == 0, -1);
-            ret = xmlSecOpenSSLEvpBlockCipherCtxFinal(ctx, out,
-                                            xmlSecTransformGetName(transform),
-                                            transformCtx);
+            ret = xmlSecOpenSSLEvpBlockCipherCtxFinal(ctx, in, out,
+                    xmlSecTransformGetName(transform),
+                    transformCtx);
             if(ret < 0) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                            "xmlSecOpenSSLEvpBlockCipherCtxFinal",
-                            XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                            XMLSEC_ERRORS_NO_MESSAGE);
+                xmlSecInternalError("xmlSecOpenSSLEvpBlockCipherCtxFinal",
+                        xmlSecTransformGetName(transform));
                 return(-1);
             }
             transform->status = xmlSecTransformStatusFinished;
+
+            /* by now there should be no input */
+            xmlSecAssert2(xmlSecBufferGetSize(in) == 0, -1);
         }
     } else if(transform->status == xmlSecTransformStatusFinished) {
         /* the only way we can get here is if there is no input */
@@ -676,11 +642,7 @@ xmlSecOpenSSLEvpBlockCipherExecute(xmlSecTransformPtr transform, int last, xmlSe
         /* the only way we can get here is if there is no enough data in the input */
         xmlSecAssert2(last == 0, -1);
     } else {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_STATUS,
-                    "status=%d", transform->status);
+        xmlSecInvalidTransfromStatusError(transform);
         return(-1);
     }
 
